from elasticsearch import Elasticsearch, RequestsHttpConnection
from impute import Impute

from tools import *
from random import randint
from datetime import datetime
import urllib3
import csv
import os
import sys
import argparse
import time
import pandas as pd
from matplotlib.offsetbox import TextArea, VPacker, AnnotationBbox
import matplotlib.patches as patches
import matplotlib.dates as mdates


def plot_test_imputation(df_test, field, starts, length):
    """ Plots the time serie to test the imputation algorithm
            Parameters:
                    df_test(dataFrame): dataFrame that contains the original
                                        data and the data imputed
                    field(string): the field we want to plot
                    starts(string): starting date to the missing period
                                    to be imputed
                    length (int): number of rows of the missing period
                                    to be imputed.
    """
    fig, ax = plt.subplots(figsize=(7, 4))
    df_test["date"] = df_test.index
    df_test["date"] = pd.to_datetime(df_test["date"])
    if length < 24:
        df_test["date"] = df_test["date"].dt.strftime("%H"+"h:"+"%M"+"m")
    else:
        df_test["date"] = df_test["date"].dt.strftime("%d/%m")
    df_test = df_test.set_index('date')
    df_test.plot(ax=ax, fontsize=11, rot=45)
    ax.set_ylabel("Temperature(°C)", fontsize=11)
    ax.set_xlabel("Date", fontsize=11)
    ax.set_title("Test of the imputation algorithm",
                 fontsize=15)
    plt.subplots_adjust(bottom=0.35, top=0.92,)
    texts = ['start_date : {}'.format(starts), 'node {} of zone {} '
             .format(field[14:16], field[4]), r'$\Delta t$: {}hours'
             .format(length)]
    Texts = []
    for t in texts:
        Texts.append(TextArea(t))
    texts_vbox = VPacker(children=Texts, pad=0, sep=0)
    ann = AnnotationBbox(texts_vbox, (0.7, -0.5), xycoords=ax.transAxes,
                         box_alignment=(0.3, .1),
                         bboxprops=dict(facecolor='white', boxstyle='round'))
    ann.set_figure(fig)
    fig.artists.append(ann)
    plt.savefig("../results_png/ impute{}hours.png".format(length))


def plot_mse_imputation(df_test, field, starts, mse):
    """
        Plot the mse values obtained by the algoirthm of
                 imputation according to number of rows
                 of the missing period to be imputed
            Parameters:
                    df_test(dataFrame): dataFrame that contains MSE and
                                        number of rows of the missing period
                                        to be imputed
                    field(string): the field we want to plot
                    mse(list): list of mse values obtained by the algoirthm of
                                imputation
    """
    fig, ax = plt.subplots(figsize=(7, 4))
    df_test = df_test.set_index(r"$\Delta t$")
    df_test.plot(ax=ax, fontsize=11, rot=75, kind='bar')
    plt.subplots_adjust(bottom=0.22)
    ax.set_title("Variation of the  MSE  of the {}  "
                 .format(field[17:]), fontsize=13)
    texts = ['start_date : {}'.format(starts), 'node {} of zone {}'.
             format(field[14:16], field[4])]
    Texts = []
    plt.yscale('log')
    plt.ylabel("MSE", fontsize=13)
    plt.xlabel(r"$\Delta t (h)$", fontsize=13)
    for t in texts:
        Texts.append(TextArea(t))
    texts_vbox = VPacker(children=Texts, pad=0, sep=0)
    ann = AnnotationBbox(texts_vbox, (0.27, 0.88), xycoords=ax.transAxes,
                         box_alignment=(0.2, .1),
                         bboxprops=dict(facecolor='white', boxstyle='round'),
                         fontsize=11)
    ann.set_figure(fig)
    fig.artists.append(ann)
    plt.savefig("../results_png/MSE{}.png".format(field))


def index(df, starts, end):
    """Calculates index of start and end in the df
        Parameters:
                    starts(string): starting date to the missing period
                                    to be imputed
                    end(string): end date to the missing period
                                    to be imputed
        Returns:
                s(int):index of starts
                lenght(list):list of index of  end
    """
    lis = np.arange(0, df.shape[0])
    df['col_index'] = np.array(lis)
    length = []
    s = int(df.loc[starts, 'col_index'])
    for i in end:
        t = int(df.loc[i, 'col_index']-s)
        length .append(t)
    df = df.drop(columns=['col_index'], inplace=True)
    return s, length


def test_impute(df):
    my_imp = Impute(df, window_size=impute_ws,
                    n_trees=impute_ntrees,
                    verbose=False)
    start_date = '2019-06-21T00:00:00'
    end_date = ['2019-06-21T02:00:00', '2019-06-21T06:00:00',
                '2019-06-21T11:00:00', '2019-06-22T00:00:00',
                '2019-06-23T00:00:00', '2019-06-28T00:00:00',
                '2019-07-04T00:00:00', '2019-07-12T00:00:00',
                '2019-07-21T00:00:00', '2019-07-30T00:00:00']
    s, length = index(df, start_date, end_date)
    list_mse = []
    for l in length:
            print("time", l)
            df_AImpute, df_BImpute, mse = my_imp.test_impute(df.columns[0],
                                                             s, l)
            list_mse.append(mse)
            df_test = pd.DataFrame({"real_values": df_BImpute,
                                    "infered_values": df_AImpute})
            plot_test_imputation(df_test, df.columns[0], start_date, l)
    df_test = pd.DataFrame({r"$\Delta t$": length, "MSE": list_mse})
    plot_mse_imputation(df_test, df.columns[0], start_date, list_mse)


if '__main__':
    impute_ws = 10
    impute_ntrees = 50
    df = pd.read_csv("../results_csv/export_test_impute.csv", encoding='latin',
                     delimiter=",", decimal=',', index_col=0,
                     parse_dates=True)
    df = df.drop(columns=['epoch_format'])
    df = df.apply(pd.to_numeric)
    test_impute(df)
